module omega_api_v3_example
  use kinds
  use omega_birdtracks
  use omega_api_v3
  implicit none
  private
  integer, parameter, public :: n_incoming = 2
  integer, parameter, public :: n_outgoing = 2
  integer, parameter, public :: n_particles = 4
  integer, parameter, public :: n_colorflows = 4
  integer, parameter, public :: n_flavor_states = 1
  integer, parameter, public :: n_spin_states = 1
  integer, parameter, public :: len_coupling_orders = 3
  integer, parameter, public :: n_coupling_orders = 2
  integer, parameter, public :: n_coupling_powers = 1
  complex(kind=default), dimension(n_coupling_powers,n_flavor_states,n_spin_states,n_colorflows), save :: amp = 0
  integer, dimension(n_particles,n_flavor_states), save :: flavor_states
  integer, dimension(n_particles,n_spin_states), save :: spin_states
  data flavor_states(:,1) / 21, 21,  2, -2 /
  data spin_states(:,1) / -1, -1, -1, -1 /
  character(len_coupling_orders), dimension(n_coupling_orders), save :: coupling_orders
  integer, dimension(n_coupling_orders,n_coupling_powers), save :: coupling_powers
  data coupling_orders / "QCD", "EW" /
  data coupling_powers(:,1) / 2, 0 /
  integer, parameter :: max_rank_inflowing = 1
  integer, parameter :: max_rank_outflowing = 1
  integer, parameter :: max_n_eps = 1
  integer, parameter :: max_n_eps_bar = 1
  integer, dimension(n_particles,n_colorflows), save :: rank_inflowing, rank_outflowing
  integer, dimension(n_colorflows), save :: n_eps, n_eps_bar
  integer, dimension(max_rank_inflowing,n_particles,n_colorflows), save :: inflowing, outflowing
  logical, dimension(n_particles,n_colorflows), save :: is_ghost
  integer, dimension(3,max_n_eps,n_colorflows), save :: eps
  integer, dimension(3,max_n_eps_bar,n_colorflows), save :: eps_bar
  character(1000), dimension(n_colorflows), public, save :: color_flows
  data rank_inflowing(:,1) / 0, 1, 0, 1 /
  data rank_outflowing(:,1) / 0, 1, 1, 0 /
  data n_eps(1) / 0 /, n_eps_bar(1) / 0 /
  data inflowing(:,1,1) / 0 /, outflowing(:,1,1) / 0 /, is_ghost(1,1) / .true. /
  data inflowing(:,2,1) / 1 /, outflowing(:,2,1) / 2 /, is_ghost(2,1) / .false. /
  data inflowing(:,3,1) / 0 /, outflowing(:,3,1) / 1 /, is_ghost(3,1) / .false. /
  data inflowing(:,4,1) / 2 /, outflowing(:,4,1) / 0 /, is_ghost(4,1) / .false. /
  data eps(:,1,1) / 0, 0, 0 /, eps_bar(:,1,1) / 0, 0, 0 /
  data color_flows(1) / "{g, ([1], [2]), ([], [1]), ([2], [])}" /
  data rank_inflowing(:,2) / 0, 1, 1, 1  /
  data rank_outflowing(:,2) / 0, 0, 0, 0 /
  data n_eps(2) / 1 /, n_eps_bar(2) / 0 /
  data inflowing(:,1,2) /  0 /, outflowing(:,1,2) / 0 /, is_ghost(1,2) / .false. /
  data inflowing(:,2,2) / -1 /, outflowing(:,2,2) / 0 /, is_ghost(2,2) / .false. /
  data inflowing(:,3,2) / -2 /, outflowing(:,3,2) / 0 /, is_ghost(3,2) / .false. /
  data inflowing(:,4,2) / -3 /, outflowing(:,4,2) / 0 /, is_ghost(4,2) / .false. /
  data eps(:,1,2) / -1, -2, -3 /, eps_bar(:,1,2) / 0, 0, 0 /
  data color_flows(2) / "{([], []), ([-1], []), ([-2], []), ([-3], []); eps(-1,-2,-3)}" /
  data rank_inflowing(:,3) / 0, 0, 0, 0  /
  data rank_outflowing(:,3) / 0, 1, 1, 1 /
  data n_eps(3) / 0 /, n_eps_bar(3) / 1 /
  data inflowing(:,1,3) / 0 /, outflowing(:,1,3) /  0 /, is_ghost(1,3) / .false. /
  data inflowing(:,2,3) / 0 /, outflowing(:,2,3) / -1 /, is_ghost(2,3) / .false. /
  data inflowing(:,3,3) / 0 /, outflowing(:,3,3) / -2 /, is_ghost(3,3) / .false. /
  data inflowing(:,4,3) / 0 /, outflowing(:,4,3) / -3 /, is_ghost(4,3) / .false. /
  data eps(:,1,3) / 0, 0, 0 /, eps_bar(:,1,3) / -1, -2, -3 /
  data color_flows(3) / "{([], []), ([], [-1]), ([], [-2]), ([], [-3]); eps(-1,-2,-3)}" /
  data rank_inflowing(:,4) / 0, 1, 1, 1  /
  data rank_outflowing(:,4) / 0, 1, 1, 1 /
  data n_eps(4) / 1 /, n_eps_bar(4) / 1 /
  data inflowing(:,1,4) /  0 /, outflowing(:,1,4) /  0 /, is_ghost(1,4) / .false. /
  data inflowing(:,2,4) / -1 /, outflowing(:,2,4) / -4 /, is_ghost(2,4) / .false. /
  data inflowing(:,3,4) / -2 /, outflowing(:,3,4) / -5 /, is_ghost(3,4) / .false. /
  data inflowing(:,4,4) / -3 /, outflowing(:,4,4) / -6 /, is_ghost(4,4) / .false. /
  data eps(:,1,4) / -1, -2, -3 /, eps_bar(:,1,4) / -4, -5, -6 /
  data color_flows(4) / "{([], []), ([-1], [-4]), ([-2], [-5]), ([-3], [-6]); eps(-1,-2,-3), eps(-4,-5,-6)}" /
  public :: load_amplitude
contains
  pure subroutine load_amplitude (a)
     type(amplitude), intent(inout) :: a
     call copy_amplitude &
        (a, n_incoming, flavor_states, spin_states, &
         rank_inflowing, inflowing, rank_outflowing, outflowing, is_ghost, &
         n_eps, eps, n_eps_bar, eps_bar, &
         coupling_orders, coupling_powers, amp)
  end subroutine load_amplitude
end module omega_api_v3_example
program test_omega_api
  use kinds
  use omega_birdtracks
  use omega_api_v3
  use omega_api_v3_example
  implicit none
  integer :: rc
  character(:), allocatable :: buffer    
  type(amplitude) :: a
  integer :: n
  call load_amplitude (a)
  do n = 1, n_colorflows
    rc = a%color_flows(n)%check ()
    if (rc /= 0) then
       print *, 'color_flow_check returned ', rc
       stop 1
    end if
    buffer = ""
    buffer = a%color_flows(n)%to_string ()
    if (trim (buffer) /= trim (color_flows(n))) then
      print *, 'color_flow%to_string failed:'
      print *, '  expected "', trim (color_flows(n)), '"'
      print *, '       got "', trim (buffer), '"'
      stop 1
    end if
  end do
end program test_omega_api
